/**
 * @file status.c
 * @brief show status
 * @author Michael Burmeister
 * @version 1.0
 * @date April 15, 2024
 * 
 */

#include <string.h>
#include "freertos/FreeRTOS.h"
#include "freertos/event_groups.h"
#include "esp_log.h"
#include "driver/gpio.h"
#include "esp_mac.h"
#include "esp_wifi.h"
#include "esp_event.h"
#include "esp_netif_net_stack.h"
#include "esp_netif.h"

#include "status.h"

/* The event group allows multiple bits for each event, but we only care about two events:
 * - we are connected to the AP with an IP
 * - we failed to connect after the maximum amount of retries */
#define WIFI_CONNECTED_BIT BIT0
#define WIFI_FAIL_BIT      BIT1


static const char* TAG = "status";
static int s_retry_num = 0;
static EventGroupHandle_t s_wifi_event_group;
/* Access Point ID of ESP32 unit */
static char SSID[] = "EZSBC"; 
static char Passwrd[] = "";
/* Your home router config */
static char Home[] = "Your Router";
static char Home_Password[] = "Router Password";
wifi_config_t config1;

wifi_config_t config2;


static void wifi_event_handler(void *arg, esp_event_base_t event_base, int32_t event_id, void *event_data)
{
    if (event_base == WIFI_EVENT && event_id == WIFI_EVENT_AP_STACONNECTED) {
        wifi_event_ap_staconnected_t *event = (wifi_event_ap_staconnected_t *) event_data;
        ESP_LOGI(TAG, "Station "MACSTR" joined, AID=%d",
                 MAC2STR(event->mac), event->aid);
    } else if (event_base == WIFI_EVENT && event_id == WIFI_EVENT_AP_STADISCONNECTED) {
        wifi_event_ap_stadisconnected_t *event = (wifi_event_ap_stadisconnected_t *) event_data;
        ESP_LOGI(TAG, "Station "MACSTR" left, AID=%d",
                 MAC2STR(event->mac), event->aid);
    } else if (event_base == WIFI_EVENT && event_id == WIFI_EVENT_STA_START) {
        esp_wifi_connect();
        ESP_LOGI(TAG, "Station started");
    } else if (event_base == IP_EVENT && event_id == IP_EVENT_STA_GOT_IP) {
        ip_event_got_ip_t *event = (ip_event_got_ip_t *) event_data;
        ESP_LOGI(TAG, "Got IP:" IPSTR, IP2STR(&event->ip_info.ip));
        s_retry_num = 0;
        xEventGroupSetBits(s_wifi_event_group, WIFI_CONNECTED_BIT);
    }
}

void wait()
{
    wifi_config_t config;

    esp_wifi_get_config(ESP_IF_WIFI_STA, &config);
    
    EventBits_t bits = xEventGroupWaitBits(s_wifi_event_group, WIFI_CONNECTED_BIT | WIFI_FAIL_BIT,
                                           pdFALSE, pdFALSE, portMAX_DELAY);

    if (bits & WIFI_CONNECTED_BIT)
    {
        ESP_LOGI(TAG, "connected to ap SSID:%s password:%s",
                 config.sta.ssid, config.sta.password);
    }
    else if (bits & WIFI_FAIL_BIT)
    {
        ESP_LOGI(TAG, "Failed to connect to SSID:%s, password:%s",
                 config.sta.ssid, config.sta.password);
    }
    else
    {
        ESP_LOGE(TAG, "UNEXPECTED EVENT");
        return;
    }
}

/* Initialize soft AP */
esp_netif_t *wifi_init_softap(void)
{
    wifi_config_t config2; // <--- uncomment this causes stack overflow

    int channel = 1;

    esp_netif_t *esp_netif_ap = esp_netif_create_default_wifi_ap();

    strcpy((char*)config2.ap.ssid, SSID);
    config2.ap.ssid_len = strlen(SSID);
    config2.ap.channel = channel;
    strcpy((char*)config2.ap.password, Passwrd);
    config2.ap.max_connection = 4;
    config2.ap.authmode = WIFI_AUTH_WPA2_PSK;
    config2.ap.pmf_cfg.required = false;

    config2.ap.authmode = WIFI_AUTH_OPEN;

    ESP_ERROR_CHECK(esp_wifi_set_config(WIFI_IF_AP, &config2));

    ESP_LOGI(TAG, "wifi_init_softap finished. SSID:%s password:%s channel:%d",
             config2.ap.ssid, config2.ap.password, channel);

    return esp_netif_ap;
}

/* Initialize wifi station */
esp_netif_t *wifi_init_sta(void)
{
    wifi_config_t config1; //  <--- uncomment this causes stack overflow and will not connect to ap

    esp_netif_t *esp_netif_sta = esp_netif_create_default_wifi_sta();

    strcpy((char*)config1.sta.ssid, Home);
    strcpy((char*)config1.sta.password, Home_Password);
    config1.sta.scan_method = WIFI_ALL_CHANNEL_SCAN;
    config1.sta.failure_retry_cnt = 3;
    config1.sta.threshold.authmode = WIFI_AUTH_WPA_WPA2_PSK;
    config1.sta.sae_pwe_h2e = WPA3_SAE_PWE_BOTH;

    ESP_ERROR_CHECK(esp_wifi_set_config(WIFI_IF_STA, &config1) );

    ESP_LOGI(TAG, "wifi_init_sta finished.");

    return esp_netif_sta;
}

void status()
{
    ESP_ERROR_CHECK(esp_netif_init());
    ESP_ERROR_CHECK(esp_event_loop_create_default());

    s_wifi_event_group = xEventGroupCreate();

    /* Register Event handler */
    ESP_ERROR_CHECK(esp_event_handler_instance_register(WIFI_EVENT,
                    ESP_EVENT_ANY_ID,
                    &wifi_event_handler,
                    NULL,
                    NULL));
    ESP_ERROR_CHECK(esp_event_handler_instance_register(IP_EVENT,
                    IP_EVENT_STA_GOT_IP,
                    &wifi_event_handler,
                    NULL,
                    NULL));

    /*Initialize WiFi */
    wifi_init_config_t cfg = WIFI_INIT_CONFIG_DEFAULT();
    ESP_ERROR_CHECK(esp_wifi_init(&cfg));

    ESP_ERROR_CHECK(esp_wifi_set_mode(WIFI_MODE_APSTA));

    ESP_LOGI(TAG, "Status Called");
}

void start()
{
    /* Start WiFi */
    ESP_ERROR_CHECK(esp_wifi_start() );

    wait();
}


void blink(int state)
{
    gpio_set_level(18, state);
    gpio_set_level(17, state);
    gpio_set_level(16, state);
}

